
/**
 * Klasse zur Verwaltung eines Rucksackproblems
 * 
 * @author bcyrill
 *
 */
public class Rucksack {

	private int [] w;
	private int [] g;
	private int g_max;
	private int gs_max; 
	private int ws_max;
	private int k;
	
	/**
	 * Rucksack Konstruktor
	 * 
	 * @param w Wert der Elemente
	 * @param g Gewicht der Elemente
	 * @param g_max maximales Gewicht
	 * @param k Anzahl der Elemente
	 */
	Rucksack (int [] w, int [] g, int g_max, int k) {
		this.w = w;
		this.g = g;
		this.g_max = g_max;
		this.ws_max = 0;
		this.gs_max = g_max;
		this.k = k;
	}
	
	/**
	 * Testproblem
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		int k = 8;
		int g_max = 15;
		int [] g = new int [] {4,7,9,2,6,1,5,8};
		int [] w = new int [] {12,15,11,10,17,16,19,11};
		int [] s = new int[k] ; 

		Rucksack Rucksack = new Rucksack(w,g,g_max,k);	
		System.out.println("Auswahl, Gewicht, Wert");
		Rucksack.solve(0,s);
		
	}
	
	/**
	 * Methode zum loesen des Rucksackproblems
	 * 
	 * @param i aktuelle Tiefe
	 * @param s Auswahl
	 */
	private void solve(int i, int[] s) {		
		// Gewicht zu gross
		int selectedWeight = this.getWeight(s);
		if(selectedWeight > this.g_max) {
			return;
		}				
		
		int selectedValue = getValue(s);
			// Wert gleich, Gewicht kleiner
			if ((selectedValue == this.ws_max) && (selectedWeight < this.gs_max)) {
				this.gs_max = selectedWeight;
				printSelection(s);
			}
			// Grsserer Wert
			else if (selectedValue > this.ws_max) {
				this.ws_max = selectedValue;
				this.gs_max = selectedWeight;
				printSelection(s);
			}
	
		// Keine weiteren Elemente
		if (i == this.k) {
			return;
		}

		// Element in den Rucksack
		s[i] = 1;
		this.solve(i + 1, s);
		
		// Element nicht in den Rucksack
		s[i] = 0;
		this.solve(i + 1,s);
			
		return;
	}
	
	/**
	 * Gibt das Gewicht der Auswahl zurueck
	 * 
	 * @param s Auswahl
	 * @return Gewicht der Auswahl
	 */
	private int getWeight(int [] s) {
		int result = 0;
		for(int i = 0; i < this.k ; i++)
			result += g[i] * s[i];
		return result;
	}
	
	/**
	 * Gibt den Wert der Auswahl zurueck
	 * 
	 * @param s Auswahl
	 * @return Wert der Auswahl
	 */
	private int getValue(int [] s) {
		int result = 0;
		for(int i = 0; i < this.k ; i++){
			result += w[i] * s[i];
		}
		return result;
	}
	
	/**
	 * Gibt die Auswahl auf der Konsole aus
	 * 
	 * @param s Auswahl
	 */
	private void printSelection(int[] s) {
		System.out.print("(");
		for (int i=0; i < k; i++) {
				System.out.print(s[i] + " ");
		}
		System.out.print("), "+getWeight(s)+", "+getValue(s));
		System.out.println();
	}	
}
