package bcyrill;

import java.util.Vector;
import java.util.Random;

import reversi.*;

public class RatingPlayer extends bcyrill.Player implements ReversiPlayer {

	private static final int[][] rating = {
			{ 2000,   30, 400, 250, 250, 400,   30, 2000 },
			{   30,   10, 100, 100, 100, 100,   10,   30 },
			{  400,  100, 350, 150, 150, 350,  100,  400 },
			{  250,  100, 150, 100, 100, 150,  100,  250 },
			{  250,  100, 150, 100, 100, 150,  100,  250 },
			{  400,  100, 350, 150, 150, 350,  100,  400 },
			{   30,   10, 100, 100, 100, 100,   10,   30 },
			{ 2000,   30, 400, 250, 250, 400,   30, 2000 } };

	public void initialize(int color, long timeout) {
		this.color = color;
		if (color == GameBoard.RED) {
			System.out.println("RatingPlayer ist Spieler RED.");
		} else if (color == GameBoard.GREEN) {
			System.out.println("RatingPlayer ist Spieler GREEN.");
		}
		this.timeout = timeout;
	}

	public Coordinates nextMove(GameBoard gb) {
		Vector pMove = movePossible(gb);

		if (pMove.size() == 0) {
			return null;
		}

		System.out.print("RatingPlayer ");
		if (color == GameBoard.RED) {
			System.out.print("(RED)");
		} else if (color == GameBoard.GREEN) {
			System.out.print("(GREEN)");
		}
		// Initialisiere einen Anfangswert
		int max_p = Integer.MIN_VALUE;
		
		Vector best = new Vector();
		for (int i = 0; i < pMove.size(); i++) {
			TextGameBoard tgb = new TextGameBoard(gb);
			if (tgb.checkMove(color, (Coordinates) pMove.get(i))) {
				tgb.makeMove(color, (Coordinates) pMove.get(i));
			}
			// Bewerte Situation
			int eval = eval(tgb);
			
			// moeglicher Zug besser oder gleich gut
			if (eval >= max_p) {
				max_p = eval;
				// Zug besser, loesche die alten Zuege
				if (eval > max_p) {
					best.clear();
				}
				// mehrere gleich gute Zuege werden gesammelt
				best.add((Coordinates) pMove.get(i));
			}

		}
		Random r = new Random();
		// Waehle einen der besten Zuege
		return (Coordinates) best.get(r.nextInt(best.size()));
	}

	public int eval(TextGameBoard gb) {

		// Situations Beurteilung auf Grund des Felder Rating
		int points = 0, h_stones = 0, o_stones = 0, stone = 0;
		try {
			for (int row = 1; row <= gb.getSize(); row++) {
				for (int col = 1; col <= gb.getSize(); col++) {
					stone = gb.getPosition(new Coordinates(row, col));
					if (stone == this.color) {
						++h_stones;
						points += rating[row - 1][col - 1];
						continue;
					}
					if (stone == (3-this.color)) {
						++o_stones;
						points -= rating[row - 1][col - 1];
						continue;
					}
				}
			}
		} catch (OutOfBoundsException e) {
			//
		}

		// Wipe-Out Beurteilung
		if (h_stones + o_stones == 64) {
			if (h_stones > 32) {
				return Integer.MAX_VALUE;
			}
			return Integer.MIN_VALUE;
		}
		return points;
	}

}
